import logging
from typing import List

from aiohttp import ClientSession

from pyelectroluxgroup.appliance import Appliance
from pyelectroluxgroup.auth import Auth
from pyelectroluxgroup.token_manager import TokenManager

_LOGGER = logging.getLogger(__name__)


class ElectroluxHubAPI:
    """Class to communicate with the ExampleHub API."""

    def __init__(self, session: ClientSession, token_manager: TokenManager):
        """Initialize the API and store the auth so we can make requests."""
        self.token_manager = token_manager
        if not self.token_manager.ensure_credentials():
            raise ValueError("Token manager is missing credentials")

        self.auth = Auth(
            session,
            "https://api.developer.electrolux.one/api/v1",
            token_manager.api_key,
            self.async_get_access_token,
        )

    async def async_get_access_token(self) -> str:
        if self.token_manager.is_token_valid():
            return self.token_manager.access_token

        try:
            _LOGGER.debug("Refreshing access token")
            response = await self.auth.request(
                "post",
                "token/refresh",
                json={"refreshToken": self.token_manager.refresh_token},
                skip_auth_headers=True,
            )

            response.raise_for_status()
        except Exception as e:
            raise ValueError(f"Failed to get access token: {e}")

        data = await response.json()
        self.token_manager.update(data["accessToken"], data["refreshToken"])

        return self.token_manager.access_token

    async def async_get_appliances(self) -> List[Appliance]:
        """Return the appliances."""
        resp = await self.auth.request("get", "appliances")
        resp.raise_for_status()
        return [
            Appliance(appliance_data, self.auth) for appliance_data in await resp.json()
        ]

    async def async_get_appliance(self, appliance_id) -> Appliance:
        """Return the appliance."""
        resp = await self.auth.request("get", f"appliances/{appliance_id}/info")
        resp.raise_for_status()
        return Appliance(await resp.json(), self.auth)



import logging
from abc import ABC, abstractmethod
from datetime import datetime, timezone

import jwt

_LOGGER = logging.getLogger(__name__)


class TokenManager(ABC):
    """Token manager class."""

    @abstractmethod
    def __init__(self, access_token: str, refresh_token: str, api_key: str):
        """Initialize the token manager."""
        self._api_key: str | None = None
        self._refresh_token = ""
        self._access_token = ""
        self.update(access_token, refresh_token, api_key)

    @property
    def access_token(self) -> str:
        """Return the access token."""
        return self._access_token

    @property
    def refresh_token(self) -> str:
        """Return the refresh token."""
        return self._refresh_token

    @property
    def api_key(self) -> str | None:
        """Return the api key."""
        return self._api_key

    @abstractmethod
    def update(self, access_token: str, refresh_token: str, api_key: str | None = None):
        """Update the tokens."""
        self._access_token = access_token
        self._refresh_token = refresh_token
        if api_key is not None:
            self._api_key = api_key

    def ensure_credentials(self) -> bool:
        """Check if the token manager has credentials."""
        if self._api_key is None:
            _LOGGER.error("API Key is missing")
            return False
        if self._access_token == "":
            _LOGGER.error("Access Token is missing")
            return False
        if self._refresh_token == "":
            _LOGGER.error("Refresh Token is missing")
            return False
        return True

    def is_token_valid(self) -> bool:
        """Check token validity"""
        try:
            payload = jwt.decode(self.access_token, options={"verify_signature": False})
            minutes_until_expiry = (
                datetime.fromtimestamp(payload["exp"], tz=timezone.utc)
                - datetime.now(timezone.utc)
            ).total_seconds() / 60
            if minutes_until_expiry < 10:
                _LOGGER.info(
                    "Access Token is about to expire in %s minutes, refreshing it",
                    minutes_until_expiry,
                )
                return False
            return True
        except jwt.ExpiredSignatureError as e:
            _LOGGER.error("Access Token is invalid - %s", e)
            return False



from typing import Any, Dict


class Area:
    """Class representing a zone in an interactive map."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the zone with data."""
        self.data = data

    @property
    def id(self) -> str:
        """Return the area ID."""
        return self.data["id"]

    @property
    def name(self) -> str:
        """Return the area name."""
        return self.data["name"]


class Zone(Area):
    """Class representing a zone in an interactive map."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the zone with data."""
        super().__init__(data)

    @property
    def type(self) -> str:
        """Return the zone type."""
        return self.data["zoneType"]

    @property
    def power_mode(self) -> str:
        """Return the power mode of the zone."""
        return self.data["powerMode"]


class Room(Area):
    """Class representing a room in a memory map."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the room with data."""
        super().__init__(data)


class Map:
    """Base class for maps for RVCs."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the map with data."""
        self.data = data

    @property
    def id(self) -> str:
        """Return the map ID."""
        return self.data["id"]

    @property
    def name(self) -> str:
        """Return the map name."""
        return self.data["name"]

    @property
    def areas(self) -> list[Area]:
        return []


class InteractiveMap(Map):
    """Class for interactive maps, used in Pure i8 and i9 RVCs."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the interactive map with data."""
        super().__init__(data)
        self.zones: list[Area] = [Zone(zone) for zone in data.get("zones") or []]

    @property
    def areas(self) -> list[Area]:
        return list(self.zones)


class MemoryMap(Map):
    """Class for memory maps, used in 700 series RVCs."""

    def __init__(self, data: Dict[str, Any]):
        """Initialize the memory map with data."""
        super().__init__(data)
        self.rooms: list[Area] = [Room(room) for room in data.get("rooms") or []]

    @property
    def areas(self) -> list[Area]:
        return list(self.rooms)



import logging
from typing import Callable

from aiohttp import ClientResponse, ClientSession

_LOGGER = logging.getLogger(__name__)


class Auth:
    """Class to make authenticated requests."""

    def __init__(
        self,
        session: ClientSession,
        host: str,
        api_key: str | None,
        async_get_access_token: Callable,
    ):
        """Initialize the auth."""
        self.session = session
        self.host = host
        self.api_key = api_key
        self.async_get_access_token = async_get_access_token

    async def request(self, method: str, path: str, **kwargs) -> ClientResponse:
        """Make a request."""
        json = kwargs.get("json", None)
        headers = kwargs.get("headers")

        if headers is None:
            headers = {}
        else:
            headers = dict(headers)

        if not kwargs.get("skip_auth_headers", None):
            headers["x-api-key"] = self.api_key

            try:
                access_token = await self.async_get_access_token()
                headers["authorization"] = f"Bearer {access_token}"
            except Exception as e:
                _LOGGER.error(f"Failed to get access token: {e}")
                raise e

        return await self.session.request(
            method, f"{self.host}/{path}", headers=headers, json=json
        )
